LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.std_logic_unsigned.all;

ENTITY FFTController IS
   PORT (
      iReset      : IN STD_LOGIC;
      iStart      : IN STD_LOGIC;
      iStateClk   : IN STD_LOGIC;
      oSampAddr   : OUT STD_LOGIC_VECTOR(10 - 1 DOWNTO 0);
      iSamp       : IN STD_LOGIC_VECTOR(16 - 1 DOWNTO 0);
      iReadAddr   : IN STD_LOGIC_VECTOR(10 - 2 DOWNTO 0);
      iReadClock  : IN STD_LOGIC;
      oPower      : OUT STD_LOGIC_VECTOR(16 - 1 DOWNTO 0);
      oExp        : OUT STD_LOGIC_VECTOR(4 - 1 DOWNTO 0);
      oDone       : OUT STD_LOGIC;
      oinstate     : OUT STD_LOGIC_VECTOR(2 DOWNTO 0)
   );
END FFTController;

ARCHITECTURE trans OF FFTController IS
   SIGNAL sink_valid    : STD_LOGIC;
   SIGNAL sink_ready    : STD_LOGIC;
   SIGNAL sink_sop      : STD_LOGIC;
   SIGNAL sink_eop      : STD_LOGIC;
   SIGNAL sink_real     : STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL sink_imag     : STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL source_real   : STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL source_imag   : STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL source_exp    : STD_LOGIC_VECTOR((4) - 1 DOWNTO 0);
   SIGNAL source_ready  : STD_LOGIC;
   SIGNAL source_valid  : STD_LOGIC;
   SIGNAL source_sop    : STD_LOGIC;
   SIGNAL source_eop    : STD_LOGIC;
   SIGNAL source_error  : STD_LOGIC_VECTOR(1 DOWNTO 0);
   SIGNAL sink_error    : STD_LOGIC_VECTOR(1 DOWNTO 0);
   SIGNAL incount       : STD_LOGIC_VECTOR((10) - 1 DOWNTO 0);
   SIGNAL outcount      : STD_LOGIC_VECTOR((10) - 1 DOWNTO 0);
   SIGNAL inshot        : STD_LOGIC;
   SIGNAL reshot        : STD_LOGIC;
   SIGNAL wren		      : STD_LOGIC;
   SIGNAL power    		: STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL realAbs   		: STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
   SIGNAL imagAbs   		: STD_LOGIC_VECTOR((16) - 1 DOWNTO 0);
	
	type state_type is (INWAIT,INSOP,INMID,INEOP);
	signal instate   : state_type;
	component FFT
		PORT (
			clk	: IN STD_LOGIC;
			reset_n	: IN STD_LOGIC;
			fftpts_in	: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
			inverse	: IN STD_LOGIC;
			sink_valid	: IN STD_LOGIC;
			sink_sop	: IN STD_LOGIC;
			sink_eop	: IN STD_LOGIC;
			sink_real	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
			sink_imag	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
			sink_error	: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
			source_ready	: IN STD_LOGIC;
			fftpts_out	: OUT STD_LOGIC_VECTOR (10 DOWNTO 0);
			sink_ready	: OUT STD_LOGIC;
			source_error	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			source_sop	: OUT STD_LOGIC;
			source_eop	: OUT STD_LOGIC;
			source_valid	: OUT STD_LOGIC;
			source_real	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
			source_imag	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
		);
	end component;
BEGIN

   source_ready <= '1';
   sink_error <= "00";
   sink_imag <=(others =>'0');
   oSampAddr <= incount;	

   PROCESS (iStart, reshot)
   BEGIN
	   IF ((iStateClk'EVENT AND iStateClk = '1') OR (reshot'EVENT AND reshot = '1')) THEN
      IF (iStart = '1') THEN
         inshot <= '1';
      ELSE 
			inshot <= '0';
		END IF;
		END IF;
   END PROCESS;
   
   
   PROCESS (iStateClk)
   BEGIN
      IF (iStateClk'EVENT AND iStateClk = '1') THEN
         IF (iReset = '1') THEN
            sink_real <= (others =>'0');
            sink_valid <= '0';
            incount <= (others =>'0');
            instate <= INSOP;
         ELSE
            CASE instate IS
               WHEN INSOP =>
                  reshot <= '0';
                  sink_sop <= '1';
                  sink_valid <= '1';
                  sink_real <= iSamp;
                  incount(0) <= '1';
						incount((10) - 1  downto 1) <=(others =>'0');
                  instate <= INMID;
               
               WHEN INMID =>
                  sink_sop <= '0';
                  sink_real <= iSamp;
                  incount <= incount + '1';
                  IF (incount = (1024 - 2)) THEN
                     instate <= INEOP;
                  ELSE
                     instate <= INMID;
                  END IF;
               
               WHEN INEOP =>
                  sink_eop <= '1';
                  sink_real <= iSamp;
                  instate <= INWAIT;
               
               WHEN INWAIT =>
                  sink_real <= (others =>'0');
                  sink_eop <= '0';
                  sink_valid <= '0';
                  IF (inshot = '1') THEN
                     reshot <= '1';
                     incount <= (others =>'0');
                     instate <= INSOP;
                  ELSE
                     instate <= INWAIT;
                  END IF;
            END CASE;
         END IF;
      END IF;
   END PROCESS;
   
	process(source_real,source_imag) begin
		if (source_real(15) = '1') then
			realAbs <= source_real XOR "1111111111111111";
		else
			realAbs <= source_real;
		end if;
		if (source_imag(15) = '1') then
			imagAbs <= source_imag XOR "1111111111111111";
		else
			imagAbs <= source_imag;
		end if;
		if (realAbs > imagAbs) then
			power <= realAbs +  imagAbs (15 downto 1);
		else
			power <= imagAbs +  realAbs (15 downto 1);
		end if;
	end process;

   PROCESS (iStateClk)
   BEGIN
      IF (iStateClk'EVENT AND iStateClk = '1') THEN
         IF (sink_sop = '1' OR iReset = '1') THEN
            oDone <= '0';
         ELSIF (source_eop = '1') THEN
            oDone <= '1';
         END IF;
         
         IF (source_valid = '1') THEN
            IF (outcount < 512) THEN
               wren <= '1';
            ELSE
               wren <= '0';
            END IF;
            
            IF (source_eop = '1') THEN
               outcount <= (others =>'0');
            ELSE
               outcount <= outcount + '1';
            END IF;
            oExp <= source_exp;
         ELSE
            wren <= '0';
         END IF;
      END IF;
   END PROCESS;


--FFTcore: FFT port map(
--	 clk=>(iStateClk),
--    reset_n=>(NOT iReset),
--    inverse=>('0'),
--    sink_valid=>(sink_valid),
--    sink_sop=>(sink_sop),
--    sink_eop=>(sink_eop),
--    sink_real=>(sink_real),
--    sink_imag=>(sink_imag),
--    sink_error=>(sink_error),
--    source_ready=>(source_ready),
--    sink_ready=>(sink_ready),
--    source_error=>(source_error),
--    source_sop=>(source_sop),
--    source_eop=>(source_eop),
--    source_valid=>(source_valid),
--    source_exp=>(source_exp),
--    source_real=>(source_real),
--    source_imag=>(source_imag)
--						);
	
END trans;
