LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.std_logic_unsigned.all;
entity VR is
	port (
				clk_sample	: in  std_logic;
				clk_fft		: in  std_logic;
				reset			: in  std_logic;
				wren			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				address_in	: in  std_logic_vector ( 10 downto 0 );
				down_slow		: out std_logic;
				up_slow		: out std_logic;
				hold			: out std_logic;
				ah_valid		: buffer std_logic;
				pah	 		: out	std_logic
	);
end VR;
	
ARCHITECTURE connection OF VR IS 

component AudioRAM
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		rdaddress		: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
		rdclock		: IN STD_LOGIC ;
		wraddress		: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
		wrclock		: IN STD_LOGIC  := '1';
		wren		: IN STD_LOGIC  := '0';
		q		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
END component;

component Ah_Generation_top 
	PORT (
				clock			: in  std_logic;
				reset			: in  std_logic;
				start			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				read_addr	: out std_logic_vector (  9 downto 0 );
				down_slow		: out std_logic;
				up_slow		: out std_logic;
				hold			: out std_logic;
				wren			: buffer std_logic
	);
end component;

component pah_gen 
	port(
		clk		 : in	std_logic;--this clock should be the clock of sampling data.
		reset	 : in	std_logic;
		data_in	 : in	std_logic_vector(15 downto 0);
		pah	 : out	std_logic
	);
end component;

component VR_control
	port(
		clk		 			: in	std_logic;--this clock should be the clock of sampling data.
		reset	 				: in	std_logic;
		address_in	 		: in	std_logic_vector(10 downto 0);
		address_sel_bit	: out std_logic;
		start	 				: out	std_logic
	);
end component;

component holdvalue
	port(
		clk		 			: in	std_logic;--this clock should be the clock of sampling data.
		reset	 				: in	std_logic;
		bit_in		 		: in	std_logic;
		bit_out	 			: out	std_logic
	);
end component;

signal data_inter : std_LOGIC_VECTOR(15 downto 0);
signal start : std_logic;
signal address_sel_bit : std_logic;
signal ram_read_addr : std_logic_vector(10 downto 0);
signal read_addr : std_logic_vector(9 downto 0);
signal up_slow_inter: std_logic;
signal down_slow_inter: std_logic;
signal pah_inter: std_logic;


begin
	ram_read_addr <= address_sel_bit & read_addr;
	RAM:AudioRAM port map( data_in, ram_read_addr, clk_fft, address_in, clk_sample, wren, data_inter);
	Control:VR_control port map(clk_sample, reset, address_in, address_sel_bit, start);
--	Ah:Ah_Generation_top port map(clk_fft, reset, start, data_inter, read_addr, down_slow, up_slow, hold, ah_valid);
--	Pah_Pah_Pah:pah_gen port map(clk_sample, reset, data_in, pah);

	Ah:Ah_Generation_top port map(clk_fft, reset, start, data_inter, read_addr, down_slow_inter, up_slow_inter, hold, ah_valid);
	Pah_Pah_Pah:pah_gen port map(clk_sample, reset, data_in, pah);
--
--	hold_pah:holdvalue port map(clk_fft,reset,pah_inter,pah);
	hold_fast:holdvalue port map(clk_fft,reset,down_slow_inter,down_slow);
	hold_slow:holdvalue port map(clk_fft,reset,up_slow_inter,up_slow);
	
end connection;
