LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.std_logic_unsigned.all;

ENTITY Ah_Generation_top IS
	PORT (
				clock			: in  std_logic;
				reset			: in  std_logic;
				start			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				read_addr	: out std_logic_vector (  9 downto 0 );
				down_slow		: out std_logic;
				up_slow		: out std_logic;
				hold			: out std_logic;
				wren			: buffer std_logic
	);
END Ah_Generation_top;

ARCHITECTURE connection OF Ah_Generation_top IS 

component QC15 
	PORT (
		data_in			: IN  STD_LOGIC_VECTOR ( 15 DOWNTO 0);
		data_out			: OUT STD_LOGIC_VECTOR ( 15 DOWNTO 0)
	);
END component;	

component FFT_top
	port (
				clock			: in  std_logic;
				reset			: in  std_logic;
				start			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				data_out		: out std_logic_vector ( 15 downto 0 );
				read_addr	: out std_logic_vector (  9 downto 0 );
				write_addr	: out std_logic_vector (  9 downto 0 );
				wren			: out std_logic;
				FFTdone		: out std_logic
			);
end component;

component AhCompare 
	PORT (
		clk			: IN  STD_LOGIC;
		addr 			: IN  STD_LOGIC_VECTOR ( 9 DOWNTO 0);
		rst			: IN  STD_LOGIC;
		data_done	: IN  STD_LOGIC;
		wren 			: IN  STD_LOGIC;
		down_slow		: OUT STD_LOGIC;
		up_slow		: OUT STD_LOGIC;
		hold			: OUT STD_LOGIC
	);
end component;

component AhGen 
	PORT (
		clk			: IN  STD_LOGIC;
		rst			: IN  STD_LOGIC;
		power			: IN  STD_LOGIC_VECTOR (15 DOWNTO 0);
		addr 			: IN  STD_LOGIC_VECTOR ( 9 DOWNTO 0);
		wren			: IN  STD_LOGIC;
		addr_out		: OUT STD_LOGIC_VECTOR ( 9 DOWNTO 0);
		data_done	: OUT STD_LOGIC
	);
end component;


signal data_out	: std_logic_vector ( 15 downto 0);
signal write_addr	: std_logic_vector ( 9 downto 0);
signal addr_out	: std_logic_vector ( 9 downto 0);
signal FFTdone		: std_logic;
signal data_done	: std_logic;
signal data_inter : std_logic_vector ( 15 downto 0);
signal noise 		: std_logic;
	BEGIN
	NC:QC15 port map(data_in, data_inter);
	FFT_block: FFT_top port map ( clock, reset, start, data_inter, data_out, read_addr, write_addr, wren, FFTdone);
	Ah_gen1:	Ahgen port map(clock, reset, data_out, write_addr, wren, addr_out, data_done);
	Ah_gen2: AhCompare port map(clock, addr_out, reset, data_done, wren, down_slow, up_slow, hold);
END connection;