library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
entity Ah_Generation_top_tb is
end Ah_Generation_top_tb;

architecture tb of Ah_Generation_top_tb is
  function sl2int (x: std_logic) return integer is
		begin
		if x='1' then 
			return 1 ;
		else 
			return 0; 
		end if;
  end;
  
  
  constant clk_time_step : time := 1 ns;
	
  signal clk   		: std_logic;
  signal reset			: std_logic;
  signal wren  		: std_logic;
  signal read_addr	: std_logic_vector ( 9 downto 0 );
  signal write_addr	: std_logic_vector ( 8 downto 0 );
  signal sink_real   : std_logic_vector (16 - 1 downto 0);
  signal sink_imag   : std_logic_vector (16 - 1 downto 0);
  signal source_real : std_logic_vector (16 - 1 downto 0);
  signal source_imag : std_logic_vector (16 - 1 downto 0);
  
  constant NUM_FRAMES_c : natural := 40;
  -- for testing purposes, the input file contains 2 frames of data of sizes
  type     fftpts_list_t is array (NUM_FRAMES_c - 1 downto 0) of natural;
  signal fftpts_array : fftpts_list_t := (1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024,1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024);
  --signal fftpts_array : fftpts_list_t := (1024, 1024);
  signal start            : std_logic;
  -- number of input frames
  signal frames_in_index   : natural range 0 to NUM_FRAMES_c := 0;
  -- number of output frames
  signal frames_out_index : natural range 0 to NUM_FRAMES_c := 0;

  signal cnt        : natural range 0 to 1024;
  signal end_test   : std_logic;
  -- signal the end of the input data stream and output data stream.
  signal end_input  : std_logic;
  signal end_output : std_logic;
  signal up_fast : std_logic;		
  signal up_slow : std_logic;
  signal hold 	  : std_logic;
  signal pah	  : std_logic;
--------------------------------------------------------------------------------------------                                      
-- FFT_top Component Declaration
--------------------------------------------------------------------------------------------                                      
component VR
	port (
				clock			: in  std_logic;
				reset			: in  std_logic;
				start			: in  std_logic;
				data_in		: in  std_logic_vector ( 15 downto 0 );
				read_addr	: out std_logic_vector (  9 downto 0 );
				up_fast		: out std_logic;
				up_slow		: out std_logic;
				hold			: out std_logic;
				wren			: buffer std_logic;
				pah	 		: out	std_logic
	);
end component;
  
begin
  -----------------------------------------------------------------------------------------------
  -- Reset Generation                                                                          
  -----------------------------------------------------------------------------------------------
  reset	 <= '1',
             '0' after 92*clk_time_step;
  -----------------------------------------------------------------------------------------------
  -- Clock Generation                                                                         
  -----------------------------------------------------------------------------------------------
  clkgen : process
  begin
    if end_test = '1' and  end_output = '1' then
      clk <= '0';
      wait;
    else
      clk <= '0';
      wait for 5*clk_time_step;
      clk <= '1';
      wait for 5*clk_time_step;
    end if;
  end process clkgen;



  -- start valid for first cycle to indicate that the file reading should start.
  start_p : process (clk, reset)
  begin
    if reset = '1' then
      start <= '1';
--    elsif rising_edge(clk) then
--      if sink_valid = '1' and sink_ready = '1' then
--        start <= '0';
--      end if;
    end if;
  end process start_p;


  -- count the input frames and increment the index
  increment_index_p : process (clk, reset)
  begin
    if reset = '1' then
      frames_in_index <= 0;
    elsif rising_edge(clk) then
      if read_addr = "1111111111" then
        if frames_in_index < NUM_FRAMES_c - 1 then
          frames_in_index <= frames_in_index + 1;
        end if;
      end if;
    end if;
  end process increment_index_p;

  -- count the output frames and increment the index
  increment_out_index_p : process (clk, reset)
  begin
    if reset = '1' then
      frames_out_index <= 0;
    elsif rising_edge(clk) then
      if wren = '0' then
        if frames_out_index < NUM_FRAMES_c - 1 then
          frames_out_index <= frames_out_index + 1;
        end if;
      end if;
    end if;
  end process increment_out_index_p;

  -- signal when all of the input data has been sent to the DUT
  end_input <= '1' when ((frames_in_index = NUM_FRAMES_c - 1)) else
               '0';

  -- signal when all of the output data has be received from the DUT
  end_output <= '1' when (frames_out_index = NUM_FRAMES_c - 1) else
                '0';

  -- halt the input when done
  end_test_p : process (clk, reset)
  begin
    if reset = '1' then
      end_test <= '0';
    elsif rising_edge(clk) then
      if end_input = '1' then
        end_test <= '1';
      end if;
    end if;
  end process end_test_p;

  -----------------------------------------------------------------------------------------------
  -- Read input data from files                                                                  
  -----------------------------------------------------------------------------------------------
  testbench_i : process(clk) is
    file r_file     : text open read_mode is "real_input.txt";
    variable data_r : integer;
    variable rdata  : line;
    file i_file     : text open read_mode is "FFT_imag_input.txt";
    variable idata  : line;
    variable data_i : integer;
  begin
    if(reset = '1') then
      sink_real  <= std_logic_vector(to_signed(0, 16));
      sink_imag  <= std_logic_vector(to_signed(0, 16));
--      sink_valid <= '0';
    elsif rising_edge(clk) then

      -- send in NUM_FRAMES_c of data or until the end of the file
      if not endfile(r_file) and (end_input = '0' and end_test = '0') then
        if (start = '1') then
          readline(r_file, rdata);
          read(rdata, data_r);
          readline(i_file, idata);
          read(idata, data_i);
--          sink_valid <= '1';
          sink_real  <= std_logic_vector(to_signed(data_r, 16));
          sink_imag  <= std_logic_vector(to_signed(data_i, 16));
        else
--          sink_valid <= '1';
          sink_real  <= sink_real;
          sink_imag  <= sink_imag;
        end if;
		  
      else
--        sink_valid <= '0';
        sink_real <= std_logic_vector(to_signed(0, 16));
        sink_imag <= std_logic_vector(to_signed(0, 16));
      end if;
    end if;
    
  end process testbench_i;

  ---------------------------------------------------------------------------------------------
  -- Write Real and Imginary Components and Block Exponent to Files                                               
  ---------------------------------------------------------------------------------------------

  testbench_o : process(clk, reset) is
    file ro_file   : text open write_mode is "ah_fast_output_vhd.txt";
    variable rdata : line;
    variable data_r : integer;
    file io_file   : text open write_mode is "ah_slow_output_vhd.txt";
    variable idata : line;
    variable data_i : integer;
    file so_file   : text open write_mode is "ah_hold_output_vhd.txt";
    variable sdata : line;
    variable data_s : integer;
  begin
    if rising_edge(clk) and reset = '0' then
      if(wren = '1') then
        data_r := sl2int(up_fast);
        data_i := sl2int(up_slow);
        data_s := sl2int(hold);
        write(rdata, data_r);
        writeline(ro_file, rdata);
        write(idata, data_i);
        writeline(io_file, idata);
        write(idata, data_s);
        writeline(io_file, sdata);
      end if;
    end if;
  end process testbench_o;
  

  
  ---------------------------------------------------------------------------------------------
  -- FFT Component Instantiation                                                               
  ---------------------------------------------------------------------------------------------
  Ah_inst : VR
    port map (
				clk,			
				reset,			
				start,			
				sink_real,		
				read_addr,	
				up_fast,		
				up_slow,		
				hold,
				wren,
				pah
				);
end tb;

