library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;


entity pah_gen is
	port(
		clk		 : in	std_logic;--this clock should be the clock of sampling data.
		reset	 : in	std_logic;
		data_in	 : in	std_logic_vector(15 downto 0);
		pah	 : out	std_logic
	);
end entity;

architecture rtl of pah_gen is
	type state_type is (s0, s1, s2, s3, s4);
	signal state   	: state_type;
	signal counter 	: std_logic_vector(9 downto 0);
	signal counter0	: std_logic_vector(9 downto 0);
	signal counter1	: std_logic_vector(9 downto 0);
	constant amplitude1_threshold1 : std_logic_vector := "1000010101110101";
	constant amplitude1_threshold2 : std_logic_vector := "0111101010000101";
	
	constant amplitude2_threshold1 : std_logic_vector := "1100001101010000";
	constant amplitude2_threshold2 : std_logic_vector := "0100111000100000";

	constant counter1_threshold : std_logic_vector := "0000010100";
	
--counter is the number of signals in a certain state, and the maximum value is 1024.
--counter0 is the number of '0' signals in a certain state.
--counter1 is the number of '1' signals in a certain state.
--The above three values will be reseted when state transision happens.
--amplitude1 is the noise treshold.
--amplitude2 is the peak treshold.
--counter1 threshold : When the amount of peaks in 1024 sampling datas is larger than this value, we think there is a "pah".

begin
	process (clk, reset)
	begin
		if reset = '1' then
			state <= s0;
			counter  <= "0000000000";
			counter0 <= "0000000000";
			counter1 <= "0000000000";
			pah <= '0';
		elsif (rising_edge(clk)) then
			case state is
----------------------------------------------		
				when s0=>
----------------------------------------------
--				amplitude1 threshold is here 
					if data_in <= amplitude1_threshold1 or data_in >= amplitude1_threshold2 then
						counter0 <= counter0 + '1';
						counter <= counter + '1';
					else
						counter <= counter + '1';
					end if;
					if counter0 = "1111111111" and counter = "1111111111" then
						state <= s1;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					else
						state <= s0;
					end if;
					pah <= '0';

----------------------------------------------
				when s1=>
----------------------------------------------
--				amplitude2 threshold is here
					if data_in >= amplitude2_threshold1 or data_in <= amplitude2_threshold2 then
						counter1 <= counter1 + '1';
						counter <= counter + '1';
					else
						counter0 <= counter0 + '1';
						counter <= counter + '1';
					end if;
--				counter1 threshold is here
					if counter1 >= counter1_threshold and counter = "1111111111" then
						state <= s2;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					else
						state <= s1;
					end if;
					pah <= '0';
					
----------------------------------------------
				when s2=>
----------------------------------------------
--				amplitude2 threshold is here
					if data_in >= amplitude2_threshold1 or data_in <= amplitude2_threshold2 then
						counter1 <= counter1 + '1';
						counter <= counter + '1';
					else
						counter0 <= counter0 + '1';
						counter <= counter + '1';
					end if;
--				counter1 threshold is here
					if counter1 >= counter1_threshold and counter = "1111111111" then
						state <= s3;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					elsif counter0 >= "1111110000" and counter = "1111111111" then
						state <= s3;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					else
						state <= s2;
					end if;
					pah <= '0';
					
----------------------------------------------
				when s3=>
----------------------------------------------
--				amplitude2 threshold is here
					if data_in >= amplitude2_threshold1 or data_in <= amplitude2_threshold2 then
						counter1 <= counter1 + '1';
						counter <= counter + '1';
					else
						counter0 <= counter0 + '1';
						counter <= counter + '1';
					end if;
--				counter1 threshold is here
					if counter1 >= counter1_threshold and counter = "1111111111" then
						state <= s0;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					elsif counter0 >= "1111110000" and counter = "1111111111" then
						state <= s4;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					else
						state <= s3;
					end if;
					pah <= '0';
	
----------------------------------------------
				when s4 =>
----------------------------------------------
--				amplitude1 threshold is here 
					if data_in <= amplitude1_threshold1 or data_in >= amplitude1_threshold2 then
						counter0 <= counter0 + '1';
						counter <= counter + '1';
					else
						counter <= counter + '1';
					end if;
					if counter0 = "1111111111" and counter = "1111111111" then
						state <= s1;
						counter  <= "0000000000";
						counter0 <= "0000000000";
						counter1 <= "0000000000";
					else
						state <= s4;
					end if;
					pah <= '1';
			end case;

		end if;

	end process;
end rtl;
